@@ -27,9 +27,16 @@ typedef unsigned short __u16;
 typedef __signed__ int __s32;
 typedef unsigned int __u32;
 
+#ifndef __LLH_TYPE_LONG_IS_64BIT__
+#define __LLH_TYPE_LONG_IS_64BIT__
+#endif
 typedef __signed__ long __s64;
 typedef unsigned long __u64;
 
+#if (defined(__STRICT_ANSI__) && !defined(inline))
+#define inline __inline__
+#endif
+
 #endif /* __ASSEMBLY__ */
 
 #endif /* _ALPHA_TYPES_H */
