@@ -28,11 +28,14 @@ typedef __signed__ int __s32;
 typedef unsigned int __u32;
 
 #ifndef __s390x__
-#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
+#if (defined(__GNUC__) && !defined(__STRICT_ANSI__)) || (__STDC_VERSION__ >= 199901L)
 typedef __signed__ long long __s64;
 typedef unsigned long long __u64;
 #endif
 #else /* __s390x__ */
+#ifndef __LLH_TYPE_LONG_IS_64BIT__
+#define __LLH_TYPE_LONG_IS_64BIT__
+#endif
 typedef __signed__ long __s64;
 typedef unsigned long __u64;
 #endif
@@ -43,6 +46,10 @@ typedef unsigned long __u64;
 typedef unsigned long addr_t; 
 typedef __signed__ long saddr_t;
 
+#if (defined(__STRICT_ANSI__) && !defined(inline))
+#define inline __inline__
+#endif
+
 #endif /* __ASSEMBLY__ */
 
 #endif /* _S390_TYPES_H */
