@@ -19,8 +19,19 @@
 
 #include <unistd.h>
 
-#define PAGE_SIZE ((unsigned long)getpagesize())
-#define PAGE_SHIFT ((PAGE_SIZE > 65536) ? -1 : ((unsigned long[]){12,13,14,-1,15,-1,-1,-1,16}[PAGE_SIZE>>13]))
+#define PAGE_SIZE (getpagesize())
+static __inline__ int getpageshift()
+{
+    int pagesize = getpagesize();
+#if (__GNUC__ >= 4) || ((__GNUC__ == 3) && (__GNUC_MINOR__ >= 4))
+    return (__builtin_clz(pagesize) ^ 31);
+#else
+    register int pageshift = -1;
+    while (pagesize) { pagesize >>= 1; pageshift++; }
+    return pageshift;
+#endif
+}
+#define PAGE_SHIFT (getpageshift())
 #define PAGE_MASK    (~(PAGE_SIZE-1))
 
 #endif /* __ASM_SH64_PAGE_H */
