@@ -42,7 +42,6 @@
  *
  *****************************************************************************/
 
-static char ixjuser_h_rcsid[] = "Id: ixjuser.h,v 4.1 2001/08/05 00:17:37 craigs Exp";
 
 #include <linux/telephony.h>
 
@@ -288,7 +287,7 @@ typedef enum {
 	G2 = hz783,
 	GS2 = hz830,
 	A2 = hz880,
-	AS2 = hz932,
+	AS2 = hz932
 } IXJ_NOTE;
 
 typedef struct {
