@@ -4,13 +4,15 @@
 #include <asm/types.h>
 #include <linux/nfs.h>
 
+struct nlm_lockowner;
+
 /*
  * NFS lock info
  */
 struct nfs_lock_info {
-	u32		state;
-	u32		flags;
-	struct nlm_host	*host;
+	__u32		state;
+	__u32		flags;
+	struct nlm_lockowner *owner;
 };
 
 /*
