@@ -1,3 +1,5 @@
+#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
 #warning "You should include <signal.h>. This time I will do it for you."
+#endif
 #include <signal.h>
 
